{
  config,
  lib,
  pkgs,
  self,
  ...
}:

{
  config = {
    hardware.graphics = {
      enable = true;
      enable32Bit = true;
    };

    # Workaround for nvidia driver ghost display
    boot.kernelParams = [ "nvidia_drm.fbdev=1" ];

    services.xserver.videoDrivers = ["nvidia"];
    hardware.nvidia = {
      modesetting.enable = true;
      powerManagement.enable = true;
      open = false;
      nvidiaSettings = true;
      package = let
        mkDriverArgs = {
          version = "575.57.08";
          sha256_64bit = "sha256-KqcB2sGAp7IKbleMzNkB3tjUTlfWBYDwj50o3R//xvI=";
          sha256_aarch64 = "sha256-VJ5z5PdAL2YnXuZltuOirl179XKWt0O4JNcT8gUgO98=";
          openSha256 = "sha256-DOJw73sjhQoy+5R0GHGnUddE6xaXb/z/Ihq3BKBf+lg=";
          settingsSha256 = "sha256-AIeeDXFEo9VEKCgXnY3QvrW5iWZeIVg4LBCeRtMs5Io=";
          persistencedSha256 = "sha256-Len7Va4HYp5r3wMpAhL4VsPu5S0JOshPFywbO7vYnGo=";
          patches = [
            (pkgs.fetchpatch {
              url = "https://raw.githubusercontent.com/rpmfusion/nvidia-kmod/020f5fabfb067150f8dd0d6e470a7a694f59eb59/Workaround-nv_vm_flags_-calling-GPL-only-code.patch";
              hash = "sha256-2WQs8WDVzNivwUSWn7t2hoduUVvmem5e+JpGt04380c=";
            })
          ];
        };
      in ( config.boot.kernelPackages.nvidiaPackages.mkDriver mkDriverArgs ).overrideAttrs (super: {
        passthru = super.passthru or {} // {
          urls = {
            x86_64 = [
              "https://download.nvidia.com/XFree86/Linux-x86_64/${mkDriverArgs.version}/NVIDIA-Linux-x86_64-${mkDriverArgs.version}.run"
              "https://us.download.nvidia.com/XFree86/Linux-x86_64/${mkDriverArgs.version}/NVIDIA-Linux-x86_64-${mkDriverArgs.version}.run"
            ];
            aarch64 = [
              "https://us.download.nvidia.com/XFree86/aarch64/${mkDriverArgs.version}/NVIDIA-Linux-aarch64-${mkDriverArgs.version}.run"
              "https://download.nvidia.com/XFree86/Linux-aarch64/${mkDriverArgs.version}/NVIDIA-Linux-aarch64-${mkDriverArgs.version}.run"
            ];
          };
          updateScript = pkgs.den-http-get-updater {
            fileLocation = ( builtins.unsafeGetAttrPos "any" { any = null; } ).file;
            previousVersion = mkDriverArgs.version;
            versionUrl = "https://raw.githubusercontent.com/aaronp24/nvidia-versions/master/nvidia-versions.txt";
            extraPackages = with pkgs; [
              coreutils
              gawk
              gnugrep
            ];
            contentParser = lib.concatStringsSep " | " [
              "echo \"$newVersion\""
              "grep current"
              "awk '{print $3}'"
              "sort -V"
              "tail -n 1"
            ];
            unpack = false;
            prefetchList = lib.map (x: {
              inherit (x) previousHash;
              unpack = x.unpack or true;
              prefetchUrlLocation = {
                file = builtins.toString self + "/outputs.nix";
                # TODO: don't use already existing NixOS configuration
                attrpath = "nixosConfigurations.main.config.hardware.nvidia.package.${x.locationAttrpath}";
              };
            }) [
              {
                previousHash = mkDriverArgs.sha256_64bit;
                locationAttrpath = "urls.x86_64";
                unpack = false;
              }
              {
                previousHash = mkDriverArgs.sha256_aarch64;
                locationAttrpath = "urls.aarch64";
                unpack = false;
              }
              {
                previousHash = mkDriverArgs.openSha256;
                locationAttrpath = "open.src.urls";
              }
              {
                previousHash = mkDriverArgs.settingsSha256;
                locationAttrpath = "settings.src.urls";
              }
              {
                previousHash = mkDriverArgs.persistencedSha256;
                locationAttrpath = "persistenced.src.urls";
              }
            ];
          };
        };
      });
    };
    nixpkgs.config.nvidia.acceptLicense = true;
  };
}
